/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann                                   *
 *   dirk.hartmann@sitter-net.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <mysql/mysql.h>


#include "dbaccess.h"
#include "config.h"

int DBAccess::instances = 0;

DBAccess::DBAccess()
{
	connected = 0;
	instances++;
}

int DBAccess::init(const std::string &host, const std::string &user, const std::string &passwd, const std::string &db)
{
	/*copy the setting for connecting*/
	dbhost = host;
	dbuser = user;
	dbpasswd = passwd;
	database = db;
	
	return init();
}

int DBAccess::init()
{
	if (!mysql_init(&dbconn)) {
		return -1;
		err_msg = "ERROR: Database init failed: ";
		err_msg += mysql_error(&dbconn);
	}
	else
		return 0;
}

int DBAccess::connectToDb()
{
	if (connected == 1) {
		err_msg = "ERROR: Could not connect to Database, you are already connected";
		return -1;
	}
	/* connect do db */
	connection = mysql_real_connect( &dbconn, dbhost.c_str(), dbuser.c_str(), dbpasswd.c_str(), database.c_str(), 0, 0, 0 );
	if (connection == NULL) {
		err_msg = "ERROR: Could not connect to Database: ";
		err_msg += mysql_error(&dbconn);
		return -1;
	}
	connected = 1;
	return 0;
}

int DBAccess::reconnectToDb()
{
	closeMysqlConnection();
	return connectToDb();
}

int DBAccess::query(const std::string & query, std::string & queryResult, const std::string &coloumnSeparator, const std::string & rowSeparator)
{
	MYSQL_RES *result;
	MYSQL_ROW row;
	
	/* initialise all strings as empty */
	queryResult = "";

	result = NULL;

	if (connected == 0) {
		err_msg = "ERROR: You must connect before you can query";
		return -1;
	}
	
	// query stuff
	mysql_query(&dbconn, query.c_str());
	
	result = mysql_use_result(&dbconn);
	if (!result) {
		err_msg = "ERROR: Empty result: ";
		err_msg += mysql_error(&dbconn);
		/* free mysql result */
		mysql_free_result(result);
		return -1;
	}
	
	unsigned int num_fields = mysql_num_fields(result);
	
	/* fetch needed rows */
	while ((row = mysql_fetch_row(result))) {
		std::string rowResult;
		
		for (unsigned int i = 0; i < num_fields; ++i) {
			if (i == 0)
				rowResult = row[i];
			rowResult += coloumnSeparator;
			rowResult += row[i];
		}
		if (queryResult.length() + rowResult.length() < MaxQueryResultSize) { // got enough to add one more row
			if (queryResult.empty())
				queryResult = rowResult;
			else
				queryResult += rowSeparator + rowResult;
		} else {
			err_msg = "MaxQueryResultSize exceded";
			/* free mysql result */
			mysql_free_result(result);
			return -1;
		}
		
	}

	/* free mysql result */
	mysql_free_result(result);
	result = NULL;
	// std::cout << queryResult << std::endl;
	return 0;
}

void DBAccess::closeMysqlConnection()
{
	/*close Connection to mysql when connected*/
	if (connected == 1) {
		mysql_close(connection);
		mysql_close(&dbconn);
		connected = 0;
	}
}

DBAccess::~DBAccess()
{
	closeMysqlConnection();
	if (instances <= 0) // can call mysql_library_end
		mysql_library_end();
}
