/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann                                   *
 *   dirk.hartmann@sitter-net.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _SMARTBEATCLIENT_H
#define _SMARTBEATCLIENT_H

#include <iostream>
#include <sys/socket.h>
#include <sys/types.h>
#include <vector>
#include <map>

#include "smartbeatlib.h"
#include "socket_access.h"
#include "dbaccess.h"
#include "gpgaccess.h"
#include "smartbeatconfig.h"

class Smartbeatclient
{
	public:
		Smartbeatclient(const std::string &configFile);
		
		void reloadConfig();
		
		void writeToLog(const std::string &msg);
		
		void forever(); // be forever the client
		
		~Smartbeatclient();
	
	private:
		void initSocket();
		void initGpg();
		void connectToDatabases();
		void clearDatabases(); // closes all database connections and clears the map
		
		/* gets the real client ip */
		int get_my_ip(std::string &ip);

		/* writes log message and sleeps */
		void write_logmessage_and_sleep(const std::string &message);

		/* sends the status message to the server */
		int send_status(bool shortStatus = false);

		/* finds out the used duskspace in % */
		std::string find_out_diskspace(const std::string &path_to_partition) const;

		/* finds out average load */
		std::string find_out_avg_load();
		
		/* finds out uptime */
		std::string find_out_uptime();
		
		/* mix message */
		std::string mix_message(const std::vector<std::string> &xmlTags) const;
		
		// process monitord disks
		std::string tagsForMonitoredDisks() const;
		
		// process custom mysql statements
		std::string tagsForMysqlQueries();
		
		// process custom commands
		std::string tagsForCustomCommands() const;
		
		// process lm sensors
		std::string tagsForSensors() const;
		
		std::string replaceSpecialCharacters(const std::string &origXmlContent) const;

		/* when connected go to nested loop */
		void nested_loop();
	
		SmartBeatConfig config;
		SocketClass *socket; //socket file descriptor
		int authed; // is the client authenticated?
		std::string my_ip;
		map<string, DBAccess *> databases;
		GPGAccess gpg;
		bool mysqlReconnect; // reconnect value when mysql fails
		std::string mysqlStatus; // reflects the status of all mysql databases
};


#endif // smartbeatclient.h
