/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann                                   *
 *   dirk.hartmann@sitter-net.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "config.h"
#include <getopt.h>
#include <iostream>
#include <signal.h>

#include "smartbeatclient.h"

Smartbeatclient *client;

void print_usage()
/* prints out the usage */
{
	std::cout << "SmartBeatClient (Deamon) " << VERSION << std::endl;
	std::cout << "usage:\t --help or -h prints out this help" << std::endl;
	std::cout << "\t --config-file [config file] or -f [config file]" << std::endl
		<< "\t \t uses the giving config file instead of the standard conig file"
		<< std::endl;
}

void catch_signals(int signal)
{
	if (signal == SIGINT) {
		std::cout << "Catched SIGINT. Terminating" << std::endl;
	}
	else if (signal == SIGTERM) {
		std::cout << "Catched SIGTERM. Terminating" << std::endl;
	}
	client->writeToLog("client successfully shutted down");
	delete client;
	exit(0);
}

void reload_config(int signal)
{
	client->reloadConfig();
}


int main(int argc, char *argv[])
{
	int c, option_index, print_help;
	std::string config_file;
	static struct option long_options[] = {
		{"config-file", 1, 0, 'f'},
		{"help", 0, 0, 'h'}
	};

	print_help = 0; // init as false
	config_file = ""; // init as empty string
	option_index = 0;
 	while (1) {
		c = getopt_long(argc, argv, "f:h", long_options, &option_index);
		if (c == -1)
			break;
		switch (c) {
			case 'f':
				if (optarg) {
					config_file = optarg;
				}
				break;
			case 'h':
				print_help = 1;
				break;
		}
	}
	if (print_help == 1) 
		print_usage();
	else {
		signal(SIGINT, catch_signals); //catch cntrl-c
		signal(SIGTERM, catch_signals); //catch term signal
		signal(SIGHUP, reload_config); //catch term signal

		client = new Smartbeatclient(config_file);
		client->forever();
		delete client;
	}
	return EXIT_SUCCESS;
}
