/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann                                   *
 *   dirk.hartmann@sitter-net.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "smartbeatconfig.h"
#include "smartbeatlogger.h"
#include "config.h"

#include <iostream>
#include <fstream>
#include <string.h>

SmartBeatConfig::SmartBeatConfig()
{
	setStdConfigParameters();
}


SmartBeatConfig::~SmartBeatConfig()
{
	delete SBLogger;
}

void SmartBeatConfig::setStdConfigParameters()
{
	port = 5000;
	pinganswer = "heartbeat"; //answer from the client
	gpg_home = "/var/lib/smartbeat/.gnupg"; //optional parameters
	gpg = "/usr/bin/gpg"; //path to gpg or gpg command
	logdir = "/var/log/smartbeat/";
	server_address = "sitter-net.de";
	get_ip_url = "http://www.sitter-net.de/cgi-bin/whatismyip.pl";
	retry_time = 60; // time till next connection retry
	timout_time = 100; // 100 secs timeout
	my_id = "0";
	my_pw = "baum";
	gpg_key = "hardware@sitter-net.de"; // key used for encryption
	config_file = SYSCONFDIR;
	config_file += "/smartbeatclient.conf";
	coloumn_separator = " ";
	row_separator = "|";
	sensor_command = "/etc/init.d/lm_sensors status";
	logMessages = false;
	
	SBLogger = new SmartBeatLogger(logdir, "smartbeatclient.log");
}

vector< string > SmartBeatConfig::splitString(const string & c1, int sep, int maxSplit) const
{
	bool cont = false;
	string tmp("");
	vector< string > ret;
	
	for (unsigned int i = 0; i < c1.length(); ++i) {
		cont = true;
		if (sep & Separated::Space)
			if( c1[i] == ' ') {
				cont = false;
			}
		if (sep & Separated::Tabbed)
			if (c1[i] == '\t') {
				cont = false;
			}
		if (sep & Separated::Line)
			if (c1[i] == '\n') {
				cont = false;
			}
		if (cont) {
			tmp += c1[i];
		} else {
			if (!tmp.empty())
				ret.push_back(tmp);
			
			if (maxSplit >= 0) {
				unsigned int max = maxSplit; // avoid unsigned comparision
				if (ret.size() + 1 >= max) {
					// max split reached
					ret.push_back(c1.substr(i, c1.length()));
					return ret;
				}
			}
			
			tmp = "";
		}
	}
	if (!tmp.empty())
		ret.push_back(tmp);
	
	return ret;
}

string SmartBeatConfig::joinString(const vector< string > & strVector) const
{
	string joinedString;
	
	for (unsigned int i = 0; i < strVector.size(); ++i) {
		if (i == 0)
			joinedString = strVector[i];
		else
			joinedString += " ";
			joinedString += strVector[i];
	}
	return joinedString;
}

bool SmartBeatConfig::readConfigFromFile(const string & fileName)
{
	string line;
	vector<string> elems;
	
	if (!fileName.empty())
		config_file = fileName; // store config file
	
	std::ifstream file (config_file.c_str()); // open file
	
	if (!file.is_open()) // could not open file
		return false;
	
	while (!file.eof()) {
		std::getline(file, line);
		line = getUncommented(line);
		if (line.size() > 0) { // line is not empty
			// use max split to avoid too long splits
			elems = splitString(line, Separated::Space|Separated::Tabbed, 10);
			setParam(elems);
		}
	}
	file.close();
	checkForEntryDBEntries();
	return true;
}

string SmartBeatConfig::getUncommented(const string & s)
{
	string tmp;
	int endPos; // for creating substrings
	
	endPos = s.find_first_of('#'); // search for first comment char
	if (endPos == 0) // whole line is a comment
		return "";
	
	if (endPos > 0) // found # comment in line
		tmp = s.substr(0, endPos);
	else // no comment in line take whole line
		tmp = s;
	
	endPos = tmp.find_first_of(';');
	if (endPos == 0) // whole line is a comment
		return "";
	
	if (endPos > 0) // found ; comment in line
		tmp = s.substr(0, endPos);
	return tmp;
	
}

void SmartBeatConfig::setParam(const vector< string > & elems)
{
	if (elems.size() < 2) // no pair
		return;
	
	if ( elems[0] == "PORT" ) port = atoi(elems[1].c_str());
	else if ( elems[0] == "PINGANSWER" ) pinganswer = atoi(elems[1].c_str());
	else if ( elems[0] == "GPG" ) gpg = elems[1];
	else if ( elems[0] == "RETRY_TIME" ) retry_time = atoi(elems[1].c_str());
	else if ( elems[0] == "GPGHOME" ) gpg_home = elems[1];
	else if ( elems[0] == "GET_IP_URL" ) get_ip_url = elems[1];
	else if ( elems[0] == "SERVER_ADDRESS" ) server_address = elems[1];
	else if ( elems[0] == "TIMEOUT_TIME" ) timout_time = atoi(elems[1].c_str());
	else if ( elems[0] == "ID" ) my_id = elems[1];
	else if ( elems[0] == "GPG_ENCRYPTION_KEY" ) gpg_key = elems[1];
	else if ( elems[0] == "PASSWORD" ) my_pw = elems[1];
	else if ( elems[0] == "SENSORS" ) {
		vector<string> quotedValues = getQuoted(joinString(elems));
		if (quotedValues.size() > 0) // check that at least one is found
			sensor_command = quotedValues[0];
	}
	else if ( elems[0] == "ROW_SEPARATOR" ) {
		vector<string> quotedValues = getQuoted(joinString(elems));
		if (quotedValues.size() > 0) // check that at least one is found
			row_separator = quotedValues[0];
	}
	else if ( elems[0] == "COLOUMN_SEPARATOR" ) {
		vector<string> quotedValues = getQuoted(joinString(elems));
		if (quotedValues.size() > 0) // check that at least one is found
			coloumn_separator = quotedValues[0];
	}
	else if ( elems[0] == "LOG_MESSAGES" ) {
		if ( toUpper(elems[1]) == "YES" )
			logMessages = true;
		else
			logMessages = false;
	} else if ( elems[0] == "LOGDIR" ) {
		/* check for trailing '/' */
		if (elems[1][elems[1].length()-1] != '/') {
			logdir = elems[1];
			logdir += "/";
			SBLogger->set_complete_log_path(logdir + "smartbeatclient.log");
		} else {
			logdir = elems[1];
			SBLogger->set_complete_log_path(logdir + "smartbeatclient.log");
		}
	} else if ( elems[0] == "ADD_MONITORED_DISK" ) {
		string diskIdentifier = elems[1];
		if (elems.size() > 2 && monitored_disks.size() < MaxMapSize) {
			string line = joinString(elems);
			// use rfind to reuse found pos for creating a substring
			// after the last quote
			int quotedPos = line.rfind('"');
			if (quotedPos > 0) { // disk name is quoted
				vector<string> quotedValues = getQuoted(joinString(elems));
				if (quotedValues.size() > 0) { // check that at least one is found
					diskIdentifier = quotedValues[0];
					string subString = line.substr(quotedPos + 1, line.length());
					vector<string> items = splitString(subString, Separated::Space, 10);
					if (items.size() > 0) { // no value
						monitored_disks.insert(make_pair(diskIdentifier, items[0]));
						SBLogger->write_to_log("INFO: adding "+ items[0] +" with name "+ diskIdentifier +" to monitored disks");
					} else {
						SBLogger->write_to_log("WARN: no path/disk specified for "+ diskIdentifier);
					}
				} else {
					SBLogger->write_to_log("WARN: empty disk identifier");
				}
			} else {
				monitored_disks.insert(make_pair(diskIdentifier, elems[2]));
				SBLogger->write_to_log("INFO: adding "+ elems[2] +" with name "+ diskIdentifier +" to monitored disks");
			}
		} else {
			SBLogger->write_to_log("INFO: adding disk failed: too many items or missing option");
		}
	} else if ( elems[0] == "ADD_SENSOR" ) {
		if (elems.size() > 2 && sensors.size() < MaxMapSize) {
			vector<string> quotedValues = getQuoted(joinString(elems));
			if (quotedValues.size() > 1) { // got both values quoted
				string sensorIdentifier = quotedValues[0];
				string sensorName = quotedValues[1];
				sensors.insert(make_pair(sensorIdentifier, sensorName));
				// log identifier with quotes
				SBLogger->write_to_log("INFO: added sensor \""+ sensorName + "\" as \""+ sensorIdentifier +"\"");
			} else if (quotedValues.size() > 0) {
				// only the last value is quoted
				// because identifier quote is optional
				string sensorIdentifier = elems[1];
				string sensorName = quotedValues[0];
				sensors.insert(make_pair(sensorIdentifier, sensorName));
				SBLogger->write_to_log("INFO: added sensor \""+ sensorName + "\" as "+ sensorIdentifier);
			} else {
				SBLogger->write_to_log("INFO: adding sensor failed: too many items or missing option");
			}
		} else {
			SBLogger->write_to_log("INFO: adding sensor failed: too many items or missing option");
		}
	} else if ( elems[0] == "ADD_CUSTOM_COMMAND" ) {
		if (elems.size() > 2 && custom_commands.size() < MaxMapSize) {
			vector<string> quotedValues = getQuoted(joinString(elems));
			if (quotedValues.size() > 1) { // both quoted
				string commandIdentifier = quotedValues[0];
				string command = quotedValues[1];
				custom_commands.insert(make_pair(commandIdentifier, command));
				// log identifier with quotes
				SBLogger->write_to_log("INFO: adding custom command: identifier \""+ commandIdentifier +"\"; command \""+ command +"\"");
			} else if (quotedValues.size() > 0) {
				// only the last value is quoted
				// because identifier quote is optional
				string commandIdentifier = elems[1];
				string command = quotedValues[0];
				custom_commands.insert(make_pair(commandIdentifier, command));
				SBLogger->write_to_log("INFO: adding custom command: identifier "+ commandIdentifier +"; command \""+ command +"\"");
			} else {
				SBLogger->write_to_log("INFO: adding custom command failed: too many items or missing option");
			}
		} else {
			SBLogger->write_to_log("INFO: adding custom command failed: too many items or missing option");
		}
	} else if ( elems[0] == "ADD_DATABASE" ) {
		// check for already existing entry
		map<string, DatabaseSettings>::iterator databaseIter = mysqlDatabaseSettings.find(elems[1]);
		if (databaseIter != mysqlDatabaseSettings.end()) {
			SBLogger->write_to_log("WARNING: "+ elems[1] +" database already exists in config");
		} else if (elems.size() > 4 && mysqlDatabaseSettings.size() < MaxMapSize) {
			DatabaseSettings settings; // create and store settings
			settings.dbHost = elems[2];
			settings.dbUser = elems[3];
			settings.dbPass = elems[4];
			
			// insert database into config
			mysqlDatabaseSettings.insert(make_pair(elems[1], settings));
			SBLogger->write_to_log("INFO: added database "+ elems[1]);
		} else {
			SBLogger->write_to_log("WARNING: invalid config for "+ elems[1] +" or too many items");
		}
	} else if ( elems[0] == "ADD_MYSQL_QUERY" ) {
		// check for existing database
		map<string, DatabaseSettings>::iterator databaseIter = mysqlDatabaseSettings.find(elems[1]);
		
		if (databaseIter == mysqlDatabaseSettings.end()) {
			SBLogger->write_to_log("WARNING: "+ elems[1] +" database does not exists in config."
				"you have to add the database before");
		} else if (elems.size() > 3 && databaseIter->second.dbQueries.size() < MaxMapSize) {
			vector<string> quotedValues = getQuoted(joinString(elems));
			if (quotedValues.size() > 1) { // identifier and query quoted
				string queryIdentifier = quotedValues[0];
				string mysqlQuery = quotedValues[1]; // get quoted part
				
				databaseIter->second.dbQueries.insert(make_pair(queryIdentifier, mysqlQuery));
				
				// log this time with quotes in the query identifier
				SBLogger->write_to_log("INFO: \""+ queryIdentifier +"\" added query \""+ mysqlQuery +"\"");
			} else if (quotedValues.size() > 0) {
				// only the last value is quoted
				// because identifier quote is optional
				string queryIdentifier = elems[2];
				string mysqlQuery = quotedValues[0]; // get quoted part
				databaseIter->second.dbQueries.insert(make_pair(queryIdentifier, mysqlQuery));
				SBLogger->write_to_log("INFO: "+ queryIdentifier +" added query \""+ mysqlQuery +"\"");
			} else {
				SBLogger->write_to_log("WARNING: "+ elems[1] +" invalid query not added");
			}
		} else {
			SBLogger->write_to_log("WARNING: "+ elems[1] +" invalid query not added or too many items");
		}
	}
}

void SmartBeatConfig::checkForEntryDBEntries()
{
	map<string, DatabaseSettings>::iterator databaseIter;
	vector<string> entriesToRemove;
	
	// collects unused databases
	for (databaseIter = mysqlDatabaseSettings.begin(); databaseIter != mysqlDatabaseSettings.end();
		databaseIter++) {
		if (databaseIter->second.dbQueries.empty()) {
			entriesToRemove.push_back(databaseIter->first);
		}
	}
	
	// now remove them from map
	for (unsigned int i = 0; i < entriesToRemove.size(); ++i) {
		databaseIter = mysqlDatabaseSettings.find(entriesToRemove[i]);
		mysqlDatabaseSettings.erase(databaseIter);
		SBLogger->write_to_log("INFO: removed unused database "+ entriesToRemove[i]);
	}
}

vector < string > SmartBeatConfig::getQuoted(const string & s) const
{
	vector <string> quotedStrings;
	string copy = s;
	bool found = true;
	
	while(found) {
		int startPos = copy.find_first_of('\"') + 1; // don't use the quote as starting value
		int endPos = copy.find_first_of('\"', startPos + 1);
		if (startPos > 0 && endPos > startPos) {
			quotedStrings.push_back(copy.substr(startPos, endPos - startPos));
			
			if ((unsigned int) endPos + 1 >= copy.length()) // end reached
				return quotedStrings;
			
			copy = copy.substr(endPos + 1, copy.length());
		} else {
			found = false;
		}
	}
	
	return quotedStrings;
}

string SmartBeatConfig::toUpper(const string & s) const
{
	string s2 = s;
	
	for (unsigned int i = 0; i < s.length(); ++i) {
		s2[i] = std::toupper(s[i]);
	}
	return s2;
}

bool SmartBeatConfig::reloadConfig()
{
	bool ret;
	monitored_disks.clear();
	SBLogger->write_to_log("Reloading Config...");
	ret = readConfigFromFile();
	SBLogger->open();
	if (ret)
		SBLogger->write_to_log("... Success");
	else
		SBLogger->write_to_log("... Failed");
	return ret;
}



void SmartBeatConfig::setSmartBeatLogger ( SmartBeatLogger* newLogger )
{
	delete SBLogger;
	SBLogger = newLogger;
}

void SmartBeatConfig::setServerPort ( int newPort )
{
	port = newPort;
}

void SmartBeatConfig::setClientPinganswer ( const string& newAnswer )
{
	pinganswer = newAnswer;
}

void SmartBeatConfig::setGpgHomeDir ( const string& newHome )
{
	gpg_home = newHome;
}


void SmartBeatConfig::setReconnectTime ( int newTime )
{
	retry_time = newTime;
}

void SmartBeatConfig::setGetMyIpUrl ( const string& newGetIpUrl )
{
	get_ip_url = newGetIpUrl;
}

void SmartBeatConfig::setSmartBeatServerAddress ( const string& newServerAddress )
{
	server_address = newServerAddress;
}

void SmartBeatConfig::setSocketTimout ( int newTimeout )
{
	timout_time = newTimeout;
}

void SmartBeatConfig::setSmartBeatId ( const string& newId )
{
	my_id = newId;
}

void SmartBeatConfig::setSmartBeatPassword ( const string& newPw )
{
	my_pw = newPw;
}

void SmartBeatConfig::setGpgKeyName ( const string& newKey )
{
	gpg_key = newKey;
}

void SmartBeatConfig::setLogDir ( const string& newLogDir )
{
	logdir = newLogDir;
}

void SmartBeatConfig::setConfigFileName ( const string& newConfigFile )
{
	config_file = newConfigFile;
}

void SmartBeatConfig::setLogSmartBeatMessages ( bool newLogState )
{
	logMessages = newLogState;
}


void SmartBeatConfig::setGpgCommand ( const string& newCommand )
{
	gpg = newCommand;
}

void SmartBeatConfig::setRowSeparator(const string & newSeparator)
{
	row_separator = newSeparator;
}

void SmartBeatConfig::setColoumnSeparator(const string & newSeparator)
{
	coloumn_separator = newSeparator;
}

void SmartBeatConfig::setSensorCommand(const string & command)
{
	sensor_command = command;
}
