/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann   *
 *   dirk.hartmann@sitter-net.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SMARTBEATLOGGER_H
#define SMARTBEATLOGGER_H

#include <iostream>
#include <fstream>
#include <pthread.h>
/**
	The SmartBeatLogger:
		every instance can handle one logfile. this instance can be accessed simultanously from various threads
*/

class SmartBeatLogger{
	
	public:
		SmartBeatLogger(); // cons for default args
		SmartBeatLogger(const std::string &path, const std::string &file); //const for path and file
		SmartBeatLogger(const std::string &completepath); // const for complete path
		
		void set_logdir(const std::string &new_logdir);
		void set_logfile(const std::string &new_logfile);
		void set_complete_log_path(const std::string &new_complete_log_path);
		
		void open(); // open the logfile

		void write_to_log(const std::string &msg, const std::string &in_extra_bracket = "");
		
		/* closes the logfile */
		inline void close(){file.close();}

		~SmartBeatLogger();
	
	private:
		void printToStderr(const std::string &msg);
	
		pthread_mutex_t logfile_mutex; // mutex for multi threading
		std::string logfile, logdir, complete_log_path; // logfile and logdir
		
		std::ofstream file; // file handler
		
		void generate_complete_logpath();

};

#endif
