/***************************************************************************
 *   Copyright (C) 2006 by Dirk Hartmann                                   *
 *   dirk.hartmann@sitter-net.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SOCKET_ACCESS_H
#define SOCKET_ACCESS_H

#include <iostream>
#include <netinet/in.h>

/* class for socket access */
class SocketClass
{
	public:
		SocketClass(const std::string &host, const int port);
		int connect();
		int close();
		int recieve(std::string &message, unsigned int newMaxMessageLength = 0, int loop = 0);
		int send(const std::string &message);

		std::string read_error() const {return error_msg;}
		void set_host(const std::string &hostname) {host = hostname;}
		void set_port(const int portnumber) {port = portnumber;}
		void set_maxMessageLength(const int new_length) {maxMessageLength = new_length;}
		void set_recv_timeout(const int time){timeout = time;} // set to 0 to disable

		std::string return_host() const {return host;}
		int return_port() {return port;}
		int return_connection_status() const {return connected;}

		~SocketClass();
	
	protected:
		enum {BufferSize = 1024};
		int sockfd; // socketfd
		int connected; // status for connection
		std::string host, error_msg;
		int port; // portnumber
		int maxMessageLength; // default 1024
		sockaddr_in addr; // struct addr
		int timeout; // recieve timeout

		int resolve_hostname(const char *host, in_addr &dst);
};

#endif //socket_access.h
